import * as cdk from 'aws-cdk-lib';
import * as apigateway from 'aws-cdk-lib/aws-apigateway';
import * as cognito from 'aws-cdk-lib/aws-cognito';
import * as lambdaNodejs from 'aws-cdk-lib/aws-lambda-nodejs';
import { Cors, ResourceOptions } from 'aws-cdk-lib/aws-apigateway';

interface ApiGatewayStackProps extends cdk.StackProps {
  userPool: cognito.UserPool;
  createContactFunction: lambdaNodejs.NodejsFunction;
  getContactFunction: lambdaNodejs.NodejsFunction;
  deleteContactFunction: lambdaNodejs.NodejsFunction;
  updateContactFunction: lambdaNodejs.NodejsFunction;
}

export class ApiGatewayStack extends cdk.Stack {
  constructor(scope: cdk.App, id: string, props: ApiGatewayStackProps) {
    super(scope, id, props);

    // Api Gateway
    const api = new apigateway.RestApi(this, 'ContactsAppApiGateway', {
      restApiName: 'ContactsAppApiGateway',
      description: 'API for managing contacts.',
    });

    cdk.Tags.of(api).add('Service', 'ApiGateway');

    // Cognito Authorizer to API
    const authorizer = new apigateway.CognitoUserPoolsAuthorizer(
      this,
      'ContactsAppAuthorizer',
      {
        cognitoUserPools: [props.userPool],
      }
    );

    // Adding CORS
    const optionsWithCors: ResourceOptions = {
      defaultCorsPreflightOptions: {
        allowOrigins: Cors.ALL_ORIGINS,
        allowMethods: Cors.ALL_METHODS,
      },
    };

    // API Routes
    const contactsResource = api.root.addResource('contacts', optionsWithCors);

    //Create Contact
    contactsResource.addMethod(
      'POST',
      new apigateway.LambdaIntegration(props.createContactFunction),
      {
        authorizer,
        authorizationType: apigateway.AuthorizationType.COGNITO,
      }
    );

    //get all Contacts
    contactsResource.addMethod(
      'GET',
      new apigateway.LambdaIntegration(props.getContactFunction),
      {
        authorizer,
        authorizationType: apigateway.AuthorizationType.COGNITO,
      }
    );

    // Get one Contact
    const singleContactResource = contactsResource.addResource(
      '{contactId}',
      optionsWithCors
    );
    singleContactResource.addMethod(
      'GET',
      new apigateway.LambdaIntegration(props.getContactFunction),
      {
        authorizer,
        authorizationType: apigateway.AuthorizationType.COGNITO,
      }
    );
    // Update one Contact
    singleContactResource.addMethod(
      'PUT',
      new apigateway.LambdaIntegration(props.updateContactFunction),
      {
        authorizer,
        authorizationType: apigateway.AuthorizationType.COGNITO,
      }
    );

    // Delete Contact
    singleContactResource.addMethod(
      'DELETE',
      new apigateway.LambdaIntegration(props.deleteContactFunction),
      {
        authorizer,
        authorizationType: apigateway.AuthorizationType.COGNITO,
      }
    );
  }
}
