import { styled } from '@mui/material/styles';
import { Box, AppBar, Toolbar, Stack, Button, Typography } from '@mui/material';
import { useTheme } from '@mui/material/styles';
import useOffSetTop from 'hooks/useOffSetTop';
import useResponsive from 'hooks/useResponsive';
import cssStyles from 'utils/cssStyles';
import { HEADER, NAVBAR } from 'config';
import { useAppDispatch, useAppSelector } from 'hooks/storeHooks';
import Logo from 'components/Logo';
import { logoutSlice } from 'storeSlices/authSlice';
import { logout } from 'authServices/logout';

// ----------------------------------------------------------------------

type RootStyleProps = {
  isCollapse: boolean;
  isOffset: boolean;
  verticalLayout: boolean;
};

const RootStyle = styled(AppBar, {
  shouldForwardProp: (prop) =>
    prop !== 'isCollapse' && prop !== 'isOffset' && prop !== 'verticalLayout',
})<RootStyleProps>(({ isCollapse, isOffset, verticalLayout, theme }) => ({
  ...cssStyles(theme).bgBlur(),
  boxShadow: 'none',
  height: HEADER.MOBILE_HEIGHT,
  zIndex: theme.zIndex.appBar + 1,
  transition: theme.transitions.create(['width', 'height'], {
    duration: theme.transitions.duration.shorter,
  }),
  [theme.breakpoints.up('lg')]: {
    height: HEADER.DASHBOARD_DESKTOP_HEIGHT,
    width: `calc(100% - ${NAVBAR.DASHBOARD_WIDTH + 1}px)`,
    ...(isCollapse && {
      width: `calc(100% - ${NAVBAR.DASHBOARD_COLLAPSE_WIDTH}px)`,
    }),
    ...(isOffset && {
      height: HEADER.DASHBOARD_DESKTOP_OFFSET_HEIGHT,
    }),
    ...(verticalLayout && {
      width: '100%',
      height: HEADER.DASHBOARD_DESKTOP_OFFSET_HEIGHT,
      backgroundColor: theme.palette.background.default,
    }),
  },
}));

// ----------------------------------------------------------------------

type Props = {
  onOpenSidebar: VoidFunction;
  isCollapse?: boolean;
  verticalLayout?: boolean;
};

export default function DashboardHeader({
  onOpenSidebar,
  isCollapse = false,
  verticalLayout = false,
}: Props) {
  const isOffset = useOffSetTop(HEADER.DASHBOARD_DESKTOP_HEIGHT);

  const isMobile = useResponsive('down', 'sm');
  const theme = useTheme();
  const dispatch = useAppDispatch();
  const currentUser = useAppSelector((state: any) => state.auth);

  const handleLogout = async () => {
    if (currentUser.tokens) {
      try {
        dispatch(logoutSlice());
        await logout(currentUser.tokens.accessToken);
        console.log('User logged out.');
      } catch (error) {
        console.error('Error logging out:', error);
      }
    }
  };

  return (
    <RootStyle
      isCollapse={isCollapse}
      isOffset={isOffset}
      verticalLayout={verticalLayout}
    >
      <Toolbar
        sx={{
          minHeight: '100% !important',
          px: { lg: 5 },
        }}
      >
        <Stack
          direction={isMobile ? 'column' : 'row'}
          alignItems='center'
          justifyContent='space-between'
          sx={{ width: '100%', mt: 1 }}
          gap={1}
        >
          <Stack
            direction={isMobile ? 'column' : 'row'}
            gap={1}
            alignItems='center'
          >
            <Logo disabledLink sx={{ width: 40, height: 40 }} />
            <Typography color={theme.palette.grey[800]} variant='h3'>
              {`Contact App`}
            </Typography>
            {currentUser && (
              <Typography color={theme.palette.grey[800]} variant='h6'>
                {!isMobile ? '- ' : ''}
                {`${currentUser.user.email}`}
              </Typography>
            )}
          </Stack>
          <Button
            sx={{ width: 'fit-content' }}
            variant='contained'
            onClick={handleLogout}
          >
            Logout
          </Button>
        </Stack>

        <Box sx={{ flexGrow: 1 }} />
      </Toolbar>
    </RootStyle>
  );
}
